@@ -1,4 +1,5 @@
 require "rexml/namespace"
+require 'rexml/text'
 
 module REXML
 	# Defines an Element Attribute; IE, a attribute=value pair, as in:
@@ -9,19 +10,22 @@ module REXML
 		include Node
 		include Namespace
 
-		attr_accessor :value			# The value of the attribute
-		attr_reader :element			# The element to which this attribute belongs
+		# The element to which this attribute belongs
+		attr_reader :element
+		# The normalized value of this attribute.  That is, the attribute with
+		# entities intact.
+		attr_writer :normalized	
 		PATTERN = /\s*(#{NAME_STR})\s*=\s*(["'])(.*?)\2/um
 
 		# Constructor.
 		#
-		# _Examples_
 		#  Attribute.new( attribute_to_clone )
 		#  Attribute.new( source )
 		#  Attribute.new( "attr", "attr_value" )
 		#  Attribute.new( "attr", "attr_value", parent_element )
 		def initialize( first, second=nil, parent=nil )
 			@element = nil
+			@normalized = true
 			if first.kind_of? Attribute
 				self.name = first.expanded_name
 				@value = first.value
@@ -34,6 +38,7 @@ module REXML
 				@element = parent if parent.kind_of? Element
 				self.name = first
 				@value = second
+				@normalized = false
 			elsif first.kind_of? Source
 				@element = second if second.kind_of? Element
 				md = first.match(PATTERN, true )
@@ -41,12 +46,10 @@ module REXML
 			else
 				raise "illegal argument #{first.type} to Attribute constructor"
 			end
-			@value = Node::read_with_substitution @value
 		end
 
 		# Returns the namespace of the attribute.
 		# 
-		# _Examples_
 		#  e = Element.new( "elns:myelement" )
 		#  e.add_attribute( "nsa:a", "aval" )
 		#  e.add_attribute( "b", "bval" )
@@ -64,7 +67,6 @@ module REXML
 
 		# Returns the namespace URL, if defined, or nil otherwise
 		# 
-		# _Examples_
 		#  e = Element.new("el")
 		#  e.add_attributes({"xmlns:ns", "http://url"})
 		#  e.namespace( "ns" )              # -> "http://url"
@@ -86,15 +88,45 @@ module REXML
 
 		# Returns this attribute out as XML source, expanding the name
 		#
-		# _Examples_
 		#  a = Attribute.new( "x", "y" )
-		#  a.to_s          # -> "x='y'"
+		#  a.to_string     # -> "x='y'"
 		#  b = Attribute.new( "ns:x", "y" )
-		#  b.to_s          # -> "ns:x='y'"
+		#  b.to_string     # -> "ns:x='y'"
+		def to_string
+			if @normalized
+				"#@expanded_name='#{@value.gsub(/'/, '&apos;')}'"
+			else
+				doc = @element.document
+				dt = doc ? doc.doc_type : nil
+				rv = "#@expanded_name='"
+				rv << Text::normalize(@value, dt)
+				rv << "'"
+				rv
+			end
+		end
+
+		# Returns the attribute value, normalized
 		def to_s
-			val = ""
-			write_with_substitution( val, @value )
-			"#@expanded_name='#{val}'"
+			if @normalized
+				@value
+			else
+				doc = @element.document
+				dt = doc ? doc.doc_type : nil
+				Text::normalize( @value, dt )
+			end
+		end
+
+		# Returns the UNNORMALIZED value of this attribute.  That is, entities
+		# that can be replaced have been replaced.
+		def value
+			return @value unless @normalized
+			doctype = nil
+			if @element
+				doc = @element.document
+				doctype = doc.doc_type if doc
+			end
+			@normalized = false
+			@value = Text::unnormalize( @value, doctype )
 		end
 
 		# Returns a copy of this attribute
@@ -120,7 +152,7 @@ module REXML
 
 		# Writes this attribute (EG, puts 'key="value"' to the output)
 		def write( output, indent=-1 )
-			output << to_s
+			output << to_string
 		end
 	end
 end
