@@ -21,6 +21,7 @@ module REXML
 		# should be nil, not supplied, or a Parent to be set as the parent 
 		# of this object
 		def initialize( first, second = nil )
+			#puts "IN COMMENT CONSTRUCTOR; SECOND IS #{second.type}"
 			super second
 			if first.kind_of? String
 				@string = first
@@ -35,7 +36,7 @@ module REXML
 			Comment.new self
 		end
 
-		def write( output, indent=-1 )
+		def write( output, indent=-1, transitive=false )
 			indent( output, indent )
 			output << START
 			output << @string
@@ -62,5 +63,9 @@ module REXML
 		def Comment.parse_stream source, listener
 			listener.comment(source.match( PATTERN, true )[1])
 		end
+
+		def Comment.pull source
+			source.match( PATTERN, true )[1]
+		end
 	end
 end
