@@ -48,6 +48,15 @@ module REXML
 			Document.new self
 		end
 
+		# According to the XML spec, a root node has no expanded name
+		def expanded_name
+			''
+			#d = doc_type
+			#d ? d.name : "UNDEFINED"
+		end
+
+		alias :name :expanded_name
+
 		# We override this, because XMLDecls and DocTypes must go at the start
 		# of the document
 		def add( child )
@@ -126,10 +135,10 @@ module REXML
 		# document will be written
 		# @param indent (optional) if given, the starting indent for the lines
 		# in the document.
-		def write( output, indent=-1 )
+		def write( output, indent=-1, transitive=false )
 			output = Output.new( output, xml_decl.encoding ) if xml_decl.encoding != "UTF-8"
 			@children.each { |node|
-				node.write( output, indent )
+				node.write( output, indent, transitive )
 				output << "\n" unless indent<0 or node == @children[-1]
 			}
 		end
@@ -161,13 +170,14 @@ module REXML
 			end
 
 			while not source.empty?
-				source.match( /^\s*/um, true )
 				word = source.match( /^\s*(<.*?)>/um )
+				source.match( /^\s*/um, true )
 				word = word[1] unless word.nil?
 				case word
 				when nil
 					word = source.match( /\s*(\S+)/um, true )
 					return if word.nil?
+					word = word[0]
 					raise "data found outside of root element ('#{word}')" if word.strip.length > 0
 				when Comment::START_RE
 					Comment.parse_stream source, listener
