@@ -42,6 +42,10 @@ module REXML
 		# * <tt>:compress_whitespace</tt> the value can be :+all+ or an array of
 		#   strings being the names of the elements to ignore whitespace on.
 		#   Overrides :+respect_whitespace+.
+		# * <tt>:ignore_whitespace_nodes</tt> the value can be :+all+ or an array
+		#   of strings being the names of the elements in which to ignore
+		#   whitespace-only nodes.  If this is set, Text nodes which contain only
+		#   whitespace will not be added to the document tree.
 		# * <tt>:raw</tt> can be :+all+, or an array of strings being the names of
 		#   the elements to process in raw mode.  In raw mode, special
 		#   characters in text is not converted to or from entities.
@@ -86,6 +90,12 @@ module REXML
 			parent.nil? ? self : parent.root
 		end
 
+		# Evaluates to the document to which this element belongs, or nil if this
+		# element doesn't belong to a document.
+		def document
+			root.parent if root
+		end
+
 		# Evaluates to +true+ if whitespace is respected for this element.  This
 		# is the case if:
 		# 1. Neither :+respect_whitespace+ nor :+compress_whitespace+ has any value
@@ -110,6 +120,17 @@ module REXML
 			@whitespace
 		end
 
+		def ignore_whitespace_nodes
+			@ignore_whitespace_nodes = false
+			if @context
+				if @context[:ignore_whitespace_nodes]
+					@ignore_whitespace_nodes = 
+						(@context[:ignore_whitespace_nodes] == :all or
+						 @context[:ignore_whitespace_nodes].include? expanded_name)
+				end
+			end
+		end
+
 		# Evaluates to +true+ if raw mode is set for this element.  This
 		# is the case if the context has :+raw+ set to :+all+ or
 		# an array containing the name of this element.
@@ -123,7 +144,7 @@ module REXML
 			@raw
 		end
 
-		once :whitespace, :raw
+		once :whitespace, :raw, :ignore_whitespace_nodes
 
 		#################################################
 		# Namespaces                                    #
@@ -136,7 +157,7 @@ module REXML
 		def prefixes
 			prefixes = []
 			prefixes = parent.prefixes if parent
-			prefixes += attributes.prefixes
+			prefixes |= attributes.prefixes
 			return prefixes
 		end
 
@@ -157,7 +178,11 @@ module REXML
 			if prefix.nil?
 				prefix = prefix()
 			end
-			prefix = "xmlns" if prefix == ""
+			if prefix == ''
+				prefix = "xmlns"
+			else
+				prefix = "xmlns:#{prefix}" unless prefix[0,5] == 'xmlns'
+			end
 			ns = attributes[ prefix ]
 			ns = parent.namespace(prefix) if ns.nil? and parent
 			return ns
@@ -172,7 +197,8 @@ module REXML
 		#   the URI
 		# Evaluates to: this Element
 		#  a = Element.new("a")
-		#  a.add_namespace("foo", "bar")
+		#  a.add_namespace("xmlns:foo", "bar" )
+		#  a.add_namespace("foo", "bar")  # shorthand for previous line
 		#  a.add_namespace("twiddle")
 		#  puts a   #-> <a xmlns:foo='bar' xmlns='twiddle'/>
 		def add_namespace( prefix, uri=nil )
@@ -180,7 +206,7 @@ module REXML
 				@attributes["xmlns"] = prefix
 			else
 				prefix = "xmlns:#{prefix}" unless prefix =~ /^xmlns:/
-				@attributes[prefix] = uri
+				@attributes[ prefix ] = uri
 			end
 			self
 		end
@@ -507,6 +533,34 @@ module REXML
 			attr.remove unless attr.nil?
 		end
 
+		#################################################
+		# Other Utilities                               #
+		#################################################
+
+		# Get an array of all CData children.  
+		# IMMUTABLE
+		def cdatas
+			find_all { |child| child.kind_of? CData }.freeze
+		end
+
+		# Get an array of all Comment children.
+		# IMMUTABLE
+		def comments
+			find_all { |child| child.kind_of? Comment }.freeze
+		end
+
+		# Get an array of all Instruction children.
+		# IMMUTABLE
+		def instructions
+			find_all { |child| child.kind_of? Instruction }.freeze
+		end
+
+		# Get an array of all Text children.
+		# IMMUTABLE
+		def texts
+			find_all { |child| child.kind_of? Text }.freeze
+		end
+
 		# Writes out this element, and recursively, all children.
 		# writer:: A String or IO (or any object supporting <<(String))
 		# indent:: 
@@ -516,8 +570,8 @@ module REXML
 		#  out = ''
 		#  doc.write( out )     #-> doc is written to the string 'out'
 		#  doc.write( $stdout ) #-> doc written to the console
-		def write(writer, indent=-1)
-			indent writer, indent
+		def write(writer, indent=-1, transitive=false)
+			#print "ID:#{indent}"
 			writer << "<#@expanded_name"
 
 			@attributes.each_attribute do |attr|
@@ -528,10 +582,19 @@ module REXML
 			if @children.empty?
 				writer << "/" 
 			else
+				if transitive and indent>-1 and !@children[0].kind_of? Text
+					writer << "\n"
+					indent writer, indent+2
+				end
 				writer << ">"
-				write_children writer, indent
+				write_children writer, indent, transitive
 				writer << "</#{expanded_name}"
 			end
+			if transitive and indent>-1
+				writer << "\n"
+				indent -= 2 if next_sibling.nil?
+				indent writer, indent
+			end
 			writer << ">"
 		end
 
@@ -574,21 +637,24 @@ module REXML
 						when Instruction::START_RE
 							Instruction.parse_stream source, listener
 						else
+							raise ParseException.new("malformed XML: bad content", source ) if md[1][0] == ?<
 							Text.parse_stream source, listener
 						end
 					end
-					if source.match(Regexp.new("^\s*<\/#{name}\s*>", Regexp::MULTILINE, 'u'), true)
+					#nregexp = Regexp.new("^\s*<\/#{name}\s*>", Regexp::MULTILINE, 'u')
+					#if source.match(nregexp, true)
+					if source.match(/^\s*<\/#{name}\s*>/, true)
 						listener.tag_end name
 					else
 						raise ParseException.new( "Missing end tag for #{name}", 
-						source, self )
+						source )
 					end
 				else
 					listener.tag_end name
 				end
 			rescue ParseException
 				$!.source = source
-				$!.element = self
+				$!.element = self if self.kind_of? Element
 				raise
 			rescue Exception
 				raise
@@ -613,6 +679,7 @@ module REXML
 		#
 		# Returns:: an array with [tag_name_S, closed_B, attributes_A]
 		TAG_MATCH = /^<(#{NAME_STR})\s*((((["']).*?\5)|[^\/'">]*)*?)(\/)?>/um
+		CLOSE_MATCH = /^\s*<\/(#{NAME_STR})\s*>/um
 		def Element.base_parser source
 			# Get the next tag
 			md = source.match(TAG_MATCH, true)
@@ -630,12 +697,16 @@ module REXML
 			# Get the next tag
 			self.name, closed, attrs = Element.base_parser(source)
 			attrs.each do |set|
-				@attributes.add( Attribute.new( set[0], set[2] ) )
+				a = Attribute.new( set[0], set[2] )
+				a.normalized = true
+				@attributes[set[0]] = a
 			end
 
 			unless closed
 				parse_children source
-				raise ParseException.new( "Missing end tag for #{expanded_name}", source, self) unless source.match(/^\s*<\/#@expanded_name\s*>/um, true)
+				md = source.match( CLOSE_MATCH, true )
+				raise ParseException.new( "Missing end tag for #{expanded_name}", 
+					source, self) unless @expanded_name == md[1]
 			end
 		rescue ParseException
 			$!.source = source
@@ -649,43 +720,56 @@ module REXML
 		# A private helper method
 		def parse_children source
 			while true
-				md = source.match(/\A(\s*[^>]*>)/um)
-				#puts "SOURCE BUFFER = #{source.buffer}, #{source.buffer.size}"
-				raise ParseException.new( "missing tag start", source, self ) unless md
-				case md[1]
-				when nil
-					return
-				when /\A<[\w_:]/um
-					Element.new(source, self, @context)
-				when /\A<\//um
-					return
-				when CData::START_RE
-					add(CData.new(source))
-				when Comment::START_RE
-					add(Comment.new(source))
-				when Instruction::START_RE
-					add(Instruction.new(source))
+				source.read if source.buffer.size==0
+				if source.buffer[0] == ?<
+					if source.buffer[1] == ?/
+						return
+					elsif source.buffer[1] == ?!
+						md = source.match(/\A(\s*[^>]*>)/um)
+						#puts "SOURCE BUFFER = #{source.buffer}, #{source.buffer.size}"
+						raise ParseException.new("Malformed node",source,self) unless md
+						case md[1]
+						when CData::START_RE
+							add(CData.new(source))
+						when Comment::START_RE
+							add(Comment.new(source))
+						else
+							raise ParseException.new("malformed XML: bad content", source, self )
+						end
+					elsif source.buffer[1] == ??
+						add(Instruction.new(source))
+					else
+						Element.new(source,self, @context)
+					end
 				else
-					add(Text.new(source, whitespace(), raw()))
+					t = Text.new(source,whitespace(),raw())
+					add(t) unless ignore_whitespace_nodes and t.empty?
 				end
 			end
 		end
 
 		# A private helper method
-		def write_children( writer, indent )
-			cr = (indent < 0) ? "" : "\n"
+		def write_children( writer, indent, transitive )
+			cr = (indent < 0) ? '' : "\n"
 			#if size == 1 and @children[0].kind_of?(Text)
 			#	self[0].write( writer, -1 )
 			if indent == -1
-				each { |child| child.write( writer, indent ) }
+				each { |child| child.write( writer, indent, transitive ) }
 			else
 				next_indent = indent+2
+				last_child=nil
 				each { |child|
-					writer << cr
-					child.write( writer, next_indent )
+					unless child.kind_of? Text or last_child.kind_of? Text or transitive
+						writer << cr
+						indent writer, next_indent
+					end
+					child.write( writer, next_indent, transitive )
+					last_child = child
 				}
-				writer << cr
-				indent( writer, indent )
+				unless last_child.kind_of? Text or transitive
+					writer << cr
+					indent( writer, indent )
+				end
 			end
 		end
 	end
@@ -739,10 +823,11 @@ module REXML
 					(num += 1) == index
 				}
 			else
-				XPath::each(@element, index ) { |element| 
-					return element if element.kind_of? Element
-				}
-				return nil
+				return XPath::first( @element, index )
+				#{ |element| 
+				#	return element if element.kind_of? Element
+				#}
+				#return nil
 			end
 		end
 
@@ -981,18 +1066,24 @@ module REXML
 				return nil if name.nil?
 				# Look for prefix
 				name =~ Namespace::NAMESPLIT
-				prefix, name = $1, $2
+				prefix, n = $1, $2
 				if prefix
-					attr = fetch( name, nil )
+					attr = fetch( n, nil )
 					# check prefix
-					return nil if attr == nil
-					if attr.kind_of? Attribute
+					if attr == nil
+					elsif attr.kind_of? Attribute
 						return attr if prefix == attr.prefix
 					else
 						attr = attr[ prefix ]
 						return attr
 					end
 				end
+				if @element.document and @element.document.doc_type
+					expn = @element.expanded_name
+					expn = @element.document.doc_type.name if expn.size == 0
+					attr_val = @element.document.doc_type.attribute_of(expn, name)
+					return Attribute.new( name, attr_val ) if attr_val
+				end
 				return nil
 			end
 			if attr.kind_of? Hash
@@ -1046,6 +1137,14 @@ module REXML
 			each_attribute do |attribute|
 				ns << attribute.name if attribute.prefix == "xmlns"
 			end
+			if @element.document and @element.document.doc_type
+				expn = @element.expanded_name
+				expn = @element.document.doc_type.name if expn.size == 0
+				@element.document.doc_type.attributes_of(expn).each {
+					|attribute|
+					ns << attribute.name if attribute.prefix == 'xmlns'
+				}
+			end
 			ns
 		end
 
