@@ -44,18 +44,21 @@ module REXML
 		def Functions::id( object )
 		end
 
-		# NOT TESTED
+		# UNTESTED
 		def Functions::local_name( node_set=nil )
-			get_namespace( node_set ){|node|node.local_name}
+			get_namespace( node_set ) do |node|
+				return node.local_name 
+			end
 		end
 
-		# NOT TESTED
 		def Functions::namespace_uri( node_set=nil )
 			get_namespace( node_set ) {|node| node.namespace}
 		end
 
 		def Functions::name( node_set=nil )
-			get_namespace( node_set ) { |node| node.name }
+			get_namespace( node_set ) do |node| 
+				node.expanded_name
+			end
 		end
 
 		# Helper method.
@@ -63,10 +66,12 @@ module REXML
 			if node_set == nil
 				yield @@node if @@node.kind_of? Namespace
 			else	
-				return "" unless node_set.kind_of? Enumerable
-				node = node_set.find{|node| node.kind_of? Namespace}
-				return "" unless node
-				yield node
+				if node_set.kind_of? Namespace
+					yield node_set
+				else
+					return unless node_set.kind_of? Enumerable
+					node_set.each { |node| yield node if node.kind_of? Namespace }
+				end
 			end
 		end
 
@@ -106,7 +111,7 @@ module REXML
 		# An object of a type other than the four basic types is converted to a
 		# string in a way that is dependent on that type.
 		def Functions::string( object=nil )
-			object = context unless object
+			#object = @context unless object
 			if object.kind_of? Array
 				string( object[0] )
 			elsif object.kind_of? Text
@@ -128,9 +133,9 @@ module REXML
 			string(string).index(string(test)) == 0
 		end
 
-		# UNTESTED
+		# Fixed by Mike Stok
 		def Functions::contains( string, test )
-			string.include? test
+			string(string).include? string(test)
 		end
 
 		# UNTESTED
@@ -143,15 +148,43 @@ module REXML
 			string[ string.index(test)+1..-1 ]
 		end
 
-		# UNTESTED
+		# Take equal portions of Mike Stok and Sean Russell; mix vigorously, and
+		# pour into a tall, chilled glass.  Serves 10,000.
 		def Functions::substring( string, start, length=0 )
-			length = string.size if length==0
-			string[start-1,length]
+			ruby_string = string(string)
+			ruby_length = if length.nil? 
+											string.length
+										else
+											number(length)
+										end
+			ruby_start = number(start)
+
+			# Handle the special cases
+			return '' if (
+				ruby_length.nan? or 
+				ruby_start.nan? or
+				ruby_start.infinite?
+			)
+
+			infinite_length = ruby_length.infinite? == 1
+			ruby_length = ruby_string.length if infinite_length
+				
+			# Now, get the bounds.  The XPath bounds are 1..length; the ruby bounds 
+			# are 0..length.  Therefore, we have to offset the bounds by one.
+			ruby_start = ruby_start.round - 1
+			ruby_length = ruby_length.round
+
+			if ruby_start < 0
+			 ruby_length += ruby_start unless infinite_length
+			 ruby_start = 0
+			end
+			return '' if ruby_length <= 0
+			ruby_string[ruby_start,ruby_length]
 		end
 
 		# UNTESTED
 		def Functions::string_length( string )
-			string.size
+			string(string).length
 		end
 
 		# UNTESTED
@@ -164,9 +197,41 @@ module REXML
 			end
 		end
 
-		# UNTESTED
+		# This is entirely Mike Stok's beast
 		def Functions::translate( string, tr1, tr2 )
-			string.tr tr1,tr2
+			from = string(tr1)
+			to = string(tr2)
+
+			# the map is our translation table.
+			#
+			# if a character occurs more than once in the
+			# from string then we ignore the second &
+			# subsequent mappings
+			#
+			# if a charactcer maps to nil then we delete it
+			# in the output.  This happens if the from
+			# string is longer than the to string
+			#
+			# there's nothing about - or ^ being special in
+			# http://www.w3.org/TR/xpath#function-translate
+			# so we don't build ranges or negated classes
+
+			map = Hash.new
+			0.upto(from.length - 1) { |pos|
+				from_char = from[pos]
+				unless map.has_key? from_char
+					map[from_char] = 
+					if pos < to.length
+						to[pos]
+					else
+						nil
+					end
+				end
+			}
+
+			string(string).unpack('U*').collect { |c|
+				if map.has_key? c then map[c] else c end
+			}.compact.pack('U*')
 		end
 
 		# UNTESTED
@@ -180,7 +245,7 @@ module REXML
 			elsif object.kind_of? Array
 				object = object.find{|x| x and true}
 			end
-			return (object and true) & true		# if rv is nil, this turns it into false
+			return object ? true : false
 		end
 
 		# UNTESTED
@@ -229,17 +294,18 @@ module REXML
 		# an object of a type other than the four basic types is converted to a
 		# number in a way that is dependent on that type
 		def Functions::number( object=nil )
-			object = @node unless object
+			object = @@node unless object
 			if object == true
-				object = "1"
+				Float(1)
 			elsif object == false
-				object = "0"
+				Float(0)
 			elsif object.kind_of? Array
-				object = string( object )
+				string( object ).to_f
+			elsif object.kind_of? Float
+				object
 			else
-				object = object.to_s
+				object.to_s.to_f
 			end
-			object.to_f
 		end
 
 		def Functions::sum( nodes )
@@ -261,6 +327,7 @@ module REXML
 		end
 
 		def Functions::method_missing( id )
+			puts "METHOD MISSING #{id.id2name}"
 			XPath.match( @@node, id.id2name )
 		end
 	end
