@@ -1,3 +1,5 @@
+require 'rexml/xmltokens'
+
 module REXML
 	# Adds named attributes to an object.
 	module Namespace
@@ -5,8 +7,7 @@ module REXML
 		attr_reader :name, :expanded_name
 		# The expanded name of the object, valid if name is set
 		attr_accessor :prefix
-		NCNAME_STR= '[\w_:][-\w\d._]*'
-		NAME_STR= "(?:#{NCNAME_STR}:)?#{NCNAME_STR}"
+		include XMLTokens
 		NAMESPLIT = /^(?:(#{NCNAME_STR}):)?(#{NCNAME_STR})/u
 
 		# Sets the name and the expanded name
