@@ -15,12 +15,10 @@ module REXML
 		end
 
 		def add( object )
-			begin
-				object.parent = self
-				@children << object
-			rescue Exception
-				raise "#{$!.message}: only Child-ren can be added to a Parent"
-			end
+			#puts "PARENT GOTS #{size} CHILDREN"
+			object.parent = self
+			@children << object
+			#puts "PARENT NOW GOTS #{size} CHILDREN"
 			object
 		end
 
