@@ -41,7 +41,7 @@ module REXML
 			XMLDecl.new(self)
 		end
 
-		def write writer, indent=-1
+		def write writer, indent=-1, transitive=false
 			indent( writer, indent )
 			writer << START.sub(/\\/u, '')
 			writer << " #{content}"
@@ -62,6 +62,10 @@ module REXML
 		end
 
 		def XMLDecl.parse_stream source, listener
+			listener.xmldecl(*pull(source))
+		end
+
+		def XMLDecl.pull source
 			results = source.match( PATTERN, true )[1]
 			version = VERSION_RE.match( results )
 			version = version[1] unless version.nil?
@@ -69,8 +73,7 @@ module REXML
 			encoding = encoding[1] unless encoding.nil?
 			standalone = STANDALONE_RE.match(results)
 			standalone = standalone[1] unless standalone.nil?
-
-			listener.xmldecl version, encoding, standalone
+			[version, encoding, standalone]
 		end
 
 		def encoding=( value )
@@ -80,7 +83,7 @@ module REXML
 			Source.val_encoding @encoding
 		end
 
-		alias :standalone? :standalone
+		alias :stand_alone? :standalone
 
 		private
 		def content
