@@ -12,7 +12,7 @@ module REXML
 		# 	The context element
 		# path::
 		# 	The xpath to search for.  If not supplied or nil, returns the first
-		# 	matching element.
+		# 	node matching '*'.
 		# namespaces::
 		# 	If supplied, a Hash which defines a namespace mapping.
 		#
@@ -23,21 +23,37 @@ module REXML
 			parser = XPathParser.new
 			parser.namespaces = namespaces
 			path = "*" unless path
-			parser.parse(path, [element])[0]
+			element = [element] unless element.kind_of? Array
+			parser.parse(path, element, true)[0]
 		end
 
+		# Itterates over nodes that match the given path, calling the supplied
+		# block with the match.
+		# element::
+		#   The context element
+		# path::
+		#   The xpath to search for.  If not supplied or nil, defaults to '*'
+		# namespaces::
+		# 	If supplied, a Hash which defines a namespace mapping
+		#
+		#  XPath.each( node ) { |el| ... }
+		#  XPath.each( node, '/*[@attr='v']' ) { |el| ... }
+		#  XPath.each( node, 'ancestor::x' ) { |el| ... }
 		def XPath::each element, path=nil, namespaces=nil, &block
 			parser = XPathParser.new
 			parser.namespaces = namespaces
 			path = "*" unless path
-			parser.parse(path, [element]).each( &block )
+			element = [element] unless element.kind_of? Array
+			parser.parse(path, element).each( &block )
 		end
 
+		# Returns an array of nodes matching a given XPath.  
 		def XPath::match element, path=nil, namespaces=nil
 			parser = XPathParser.new
 			parser.namespaces = namespaces
 			path = "*" unless path
-			parser.parse(path,[element])
+			element = [element] unless element.kind_of? Array
+			parser.parse(path,element)
 		end
 	end
 end
