@@ -1,14 +1,30 @@
 require 'rexml/namespace'
+require 'rexml/xmltokens'
+
+# Ignore this class.  It adds a __ne__ method, because Ruby doesn't seem to
+# understand object.send( "!=", foo ), whereas it *does* understand "<", "==",
+# and all of the other comparison methods.  Stupid, and annoying, and not at
+# all POLS.
+class Object
+	def __ne__(b)
+		self != b
+	end
+end
 
 module REXML
+	# You don't want to use this class.  Really.  Use XPath, which is a wrapper
+	# for this class.  Believe me.  You don't want to poke around in here.
+	# There is strange, dark magic at work in this code.  Beware.  Go back!  Go
+	# back while you still can!
 	class XPathParser
-		LITERAL		= /^'([^']*)'|"([^"]*)"/u
-		DEBUG=true
+		include XMLTokens
+		LITERAL		= /^'([^']*)'|^"([^"]*)"/u
 
 		def initialize
 			@EQ_PROC = proc do |a,o,b|
 				o = '==' if o == '='
-				o = 'ne' if o == '!='
+				o = '__ne__' if o == '!='
+				#puts "Comparing #{a.inspect} #{o} #{b.inspect}"
 				equality_relational_compare(a,o,b)
 			end
 			@REL_PROC = method(:equality_relational_compare).to_proc
@@ -26,8 +42,8 @@ module REXML
 			@namespaces = namespaces
 		end
 
-		def parse path, nodeset, halt=false
-			#puts "path is #{path}, node is #{nodeset.type}" if DEBUG
+		def parse path, nodeset, break_and_return=false
+			#puts "path is #{path}, node is #{nodeset.type}" 
 			rv = LocationPath(path, nodeset)[1]
 			rv = [] unless rv
 			return rv
@@ -39,7 +55,7 @@ module REXML
 		#	| '/' RelativeLocationPath?
 		#	| '//' RelativeLocationPath
 		def LocationPath path, nodeset
-			#puts "LocationPath( '#{path}', #{puta nodeset} )" if DEBUG
+			#puts "LocationPath( '#{path}', #{puta nodeset} )" 
 			case path
 			when /^\/\//
 				# '//' RelativeLocationPath
@@ -65,7 +81,7 @@ module REXML
 		#	| RelativeLocationPath '//' Step
 		AXIS = /^(ancestor|ancestor-or-self|attribute|child|descendant|descendant-or-self|following|following-sibling|namespace|parent|preceding|preceding-sibling|self)::/
 		def RelativeLocationPath path, nodeset
-			#puts "RelativePath( '#{path}', #{puta nodeset} )" if DEBUG
+			#puts "RelativePath( '#{path}', #{puta nodeset} )" 
 			while path.size > 0
 				case path
 				when AXIS
@@ -83,9 +99,9 @@ module REXML
 				else
 					axis_name = 'child'
 				end
-				#puts "CALLING AXE WITH #{axis_name}, #{path}, and #{puta nodeset}" if DEBUG
+				#puts "CALLING AXE WITH #{axis_name}, #{path}, and #{puta nodeset}" 
 				path,nodeset = axe( axis_name, path, nodeset )
-				#puts "axe returned '#{path}' and #{puta nodeset}" if DEBUG
+				#puts "axe returned '#{path}' and #{puta nodeset}" 
 
 				if path.size > 0 
 					case path
@@ -96,13 +112,13 @@ module REXML
 						path = $'
 						#axis_name = 'child'
 					else
-						#puts "RETURNING #{path}" if DEBUG
+						#puts "RETURNING #{path}" 
 						return [path,nodeset]
 					end
 					#path,nodeset = axe( axis_name, path, nodeset )
 				end
 			end
-			#puts "RETURNING #{path} and #{puta nodeset}" if DEBUG
+			#puts "RETURNING #{path} and #{puta nodeset}" 
 			return [path,nodeset]
 		end
 
@@ -112,30 +128,32 @@ module REXML
 
 		def GenericExpr(path, nodeset, regex, method, &block)
 			rest, results = method.call( path, nodeset )
-			#puts "First #{method} call, results are #{puta results}" if DEBUG
-			#puts "'#{rest}' != '#{path}'" if DEBUG
+			#puts "First #{method} call, results are #{puta results}" 
+			#puts "'#{rest}' != '#{path}'? #{rest!=path}" 
 			if rest != path
 				maps = []
+				#puts "LOOKING FOR #{regex.inspect}"
 				while rest =~ regex
-					#puts "LOOKED FOR #{regex.inspect}; REST IS #{rest}" if DEBUG
+					#puts "LOOKED FOR #{regex.inspect}; REST IS #{rest}" 
 					op = $1
 					rest = $'
-					#puts "Calling #{method} with #{rest}" if DEBUG
+					#puts "Calling #{method} (operator #{op}) with #{rest}" 
 					rest, res = method.call( rest, nodeset )
-					#puts "Results #{method} are #{puta res}" if DEBUG
+					#puts "Results #{method} are #{puta res}" 
 					maps << res
-					#puts "MAPS IS NOW #{puta maps}" if DEBUG
+					#puts "MAPS IS NOW #{puta maps}" 
 				end
 				maps.each { |map| results = yield( results, op, map ) }
-				#puts "Results after join are #{puta results}" if DEBUG
+				#puts "Results after join are #{puta results}" 
 			end
+			#puts "Returning [#{rest.inspect}, #{results.inspect}]"
 			[rest, results]
 		end
 
 		#| OrExpr S 'or' S AndExpr
 		#| AndExpr
 		def OrExpr path, nodeset
-			#puts "OrExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "OrExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*( or )/, @ANDEXPR ) {|a,o,b| 
 				a = Functions::boolean(a)
 				b = Functions::boolean(b)
@@ -146,7 +164,7 @@ module REXML
 		#| AndExpr S 'and' S EqualityExpr
 		#| EqualityExpr
 		def AndExpr path, nodeset
-			#puts "AndExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "AndExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*( and )/, @EQUALITYEXPR ) {|a,o,b| 
 				a = Functions::boolean( a )
 				b = Functions::boolean( b )
@@ -157,23 +175,26 @@ module REXML
 		#| EqualityExpr ('=' | '!=')  RelationalExpr
 		#| RelationalExpr
 		def EqualityExpr path, nodeset
-			#puts "EqualityExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "EqualityExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*(=|\!=)\s*/, @RELATIONALEXPR, &@EQ_PROC)
 		end
 
 		#| RelationalExpr ('<' | '>' | '<=' | '>=') AdditiveExpr
 		#| AdditiveExpr
 		def RelationalExpr path, nodeset
-			#puts "RelationalExpr( #{path}, #{puta nodeset} )" if DEBUG
-			#puts "@REL_PROC IS A #@REL_PROC" if DEBUG
-			GenericExpr( path, nodeset, /^\s*(<|>|<=|>=)\s*/, @ADDITIVEEXPR, &@REL_PROC)
+			#puts "RelationalExpr( #{path}, #{puta nodeset} )" 
+			#puts "@REL_PROC IS A #@REL_PROC" 
+			GenericExpr( path, nodeset, /^\s*(<=|>=|<|>)\s*/, @ADDITIVEEXPR, &@REL_PROC)
 		end
 
 		#| AdditiveExpr ('+' | S '-') MultiplicativeExpr
 		#| MultiplicativeExpr
 		def AdditiveExpr path, nodeset
-			#puts "AdditiveExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "AdditiveExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*(\+| -)\s*/, @MULTIPLICATIVEEXPR) {|a,o,b| 
+				a = Functions::number(a)
+				b = Functions::number(b)
+				#puts "MATH:: #{a}(#{a.type}) #{o} #{b}(#{b.type})"
 				a.send( o.strip, b )
 			}
 		end
@@ -181,7 +202,7 @@ module REXML
 		#| MultiplicativeExpr ('*' | S ('div' | 'mod') S) UnaryExpr
 		#| UnaryExpr
 		def MultiplicativeExpr path, nodeset
-			#puts "MultiplicativeExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "MultiplicativeExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*(\*| div | mod )\s*/, @UNARYEXPR) do |a,o,b|
 				a = Functions::number(a)
 				b = Functions::number(b)
@@ -191,7 +212,7 @@ module REXML
 				when ' mod '
 					a % b
 				when ' div '
-					a.divmod(b)[0]
+					a.to_f / b.to_f
 				end
 			end
 		end
@@ -199,30 +220,36 @@ module REXML
 		#| '-' UnaryExpr
 		#| UnionExpr
 		def UnaryExpr path, nodeset
-			#puts "UnaryExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "UnaryExpr( #{path}, #{puta nodeset} )" 
 			mult = 1 if path[0] == ?-
 			while path[0] == ?-
 				mult *= -1
 				path = path[1..-1]
 			end
 			rest, results = UnionExpr( path, nodeset )
-			join( results, [mult] ) {|a,o,b| a*b} if mult
+			#puts "After UnionExpr '#{path}' and #{puta nodeset}"
+			#puts "We have #{puta results} and mult=#{mult}"
+			join( results, [mult] ) {|a,o,b| 
+				a = Functions::number(a)
+				b = Functions::number(b)
+				a * b
+			} if mult
 			[rest, results]
 		end
 
 		#| UnionExpr '|' PathExpr
 		#| PathExpr
 		def UnionExpr path, nodeset
-			#puts "UnionExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "UnionExpr( #{path}, #{puta nodeset} )" 
 			GenericExpr( path, nodeset, /^\s*(\|)\s*/, @PATHEXPR) {|a,o,b| a | b}
 		end
 
 		#| LocationPath
 		#| FilterExpr ('/' | '//') RelativeLocationPath
 		def PathExpr path, nodeset
-			#puts "PathExpr( #{path}, #{puta nodeset} )" if DEBUG
+			#puts "PathExpr( #{path}, #{puta nodeset} )" 
 			p,n = FilterExpr path, nodeset
-			#puts "After FilterExpr p is '#{p}' and n is #{puta n}" #if DEBUG
+			#puts "After FilterExpr p is '#{p}' and n is #{puta n}" 
 			return [p,n] if p.size == 0
 			# p is now '/' '/'? RelativeLocationPath
 			if p =~ /^\/\/?/
@@ -231,10 +258,10 @@ module REXML
 				else
 					p = $'
 				end
-				#puts "CALLING RELATIVELOCATIONPATH" if DEBUG
-				return RelativeLocationPath p, n
+				#puts "CALLING RELATIVELOCATIONPATH" 
+				return RelativeLocationPath(p, n)
 			end
-			return LocationPath p, nodeset if p =~ /^[\/\.\@\[\w_*]/
+			return LocationPath(p, nodeset) if p =~ /^[\/\.\@\[\w_*]/
 			return [p, n]
 		end
 
@@ -242,7 +269,7 @@ module REXML
 		#| PrimaryExpr
 		def FilterExpr path, nodeset
 			path, results = PrimaryExpr(path, nodeset)
-			#puts "Path is now '#{path}', results are #{puta results}" #if DEBUG
+			#puts "Path is now '#{path}' (#{path.type}), results are #{puta results}" 
 			path, results = Predicate(path, results) if path[0] == ?[
 			[path, results]
 		end
@@ -252,59 +279,63 @@ module REXML
 		#| LITERAL
 		#| NUMBER
 		#| FunctionCall
-		VARIABLE_REFERENCE	= /^\$(#{Namespace::NAME_STR})/u
+		VARIABLE_REFERENCE	= /^\$(#{NAME_STR})/u
 		NUMBER							= /^(\d*\.?\d+)/
 		def PrimaryExpr path, nodeset
-			#puts "PrimaryExpr('#{path}', #{puta nodeset})" if DEBUG
+			#puts "#"*5+"PrimaryExpr('#{path}', #{puta nodeset})" 
 			arry = []
 			case path
 			when VARIABLE_REFERENCE
+				#puts "VARIABLE REF"
 				varname = $1
 				path = $'
-				#nodeset.size.times { arry << @variables[ varname ] }
 				arry << @variables[ varname ]
-			when LITERAL, NUMBER
-				path = $'
-				#nodeset.size.times { arry << $1 }
-				arry << $1
-			#when NUMBER
-			#	path = $'
-			#	nodeset.size.times { arry << $1 }
 			when /^(\w[-\w]*)(?:\()/
+				#puts "GOT FUNCTION #$1"
 				fname = $1
 				path = $'
 				return nil if fname =~ NT
+				#return ['',[]] if fname =~ NT
+				#puts "Calling #{fname} with #{path}"
 				return FunctionCall(fname, path, nodeset)
+			when LITERAL, NUMBER
+				#puts "LITERAL, NUMBER = #$& > [#$1, #$2]"
+				varname = $1.nil? ? $2 : $1
+				path = $'
+				arry << varname
 			when /^\(/ 																							#/
 				path, contents = get_group path
 				contents = contents[1..-2]
-				#puts "PATH=#{path} CONTENTS=#{contents}" if DEBUG
+				#puts "PATH=#{path} CONTENTS=#{contents}" 
 				p, arry = OrExpr( contents, nodeset )
 			else
 				arry = nodeset
 			end
+			#puts "Returning '#{path}', '#{puta arry}'"
 			return [path, arry]
 		end
 
 		#| FUNCTION_NAME '(' ( expr ( ',' expr )* )? ')'
 		def FunctionCall funcname, rest, nodeset
-			#puts "FunctionCall: #{funcname}(#{rest})" if DEBUG
+			#puts "FunctionCall: #{funcname}(#{rest})" 
 			funcname.gsub!(/-/, '_')
 			path, arguments = parse_args rest
-			#puts "ARGUMENTS SIZE = #{arguments.size}" if DEBUG
+			#puts "ARGUMENTS SIZE = #{arguments.size}" 
+			#puts "ARGUMENTS = #{puta arguments}" 
 			args = arguments.collect {|arg| OrExpr( arg, nodeset )[1] }
+			#puts "ARGS IS #{puta args}"
 			results = []
 			if args.size > 0
-				#puts "NODESET IS ::: #{nodeset.size}" if DEBUG
-				#puts "CURARGS IS ::: #{args.size}" if DEBUG
+				#puts "NODESET IS ::: #{nodeset.size}" 
+				#puts "CURARGS IS ::: #{args.size}" 
 
 				nodeset.each_index do |index|
-					#puts "1) Calling #{funcname}(#{args.join ', '}) with #{args.size} args" #if DEBUG
+					#puts "1) Calling #{funcname}(#{args.join ', '}) with #{args.size} args" 
 					results << Functions.send( funcname, *args )
 				end
 			else
 				nodeset.each_index do |index|
-					#puts "\n2) Calling #{funcname}()" if DEBUG
+					#puts "\n2) Calling #{funcname}()" 
 					results << Functions.send( funcname )
 				end
 			end
@@ -312,7 +343,7 @@ module REXML
 		end
 
 		def axe( axis_name, path, nodeset ) 
-			#puts "axe #{axis_name}, #{path} #{puta nodeset}" if DEBUG
+			#puts "axe #{axis_name}, #{path} #{puta nodeset}" 
 			return unless nodeset
 
 			case axis_name
@@ -322,11 +353,11 @@ module REXML
 				results = []
 				a = ""
 				recurse( nodeset ) { |node| 
-					a, b = axe_helper(path, [node]) if node.kind_of? Element 
+					a, b = axe_helper(path, [node]) 
 					results += b if b
 				}
 				results.uniq!
-				#puts "RESULTS ARE NOW #{puta results}" if DEBUG
+				#puts "RESULTS ARE NOW #{puta results}" 
 				[a, results]
 
 			when /^ancestor/u
@@ -338,37 +369,38 @@ module REXML
 						ancestors << element
 					end
 				end
-				#puts "ancestors is #{puta ancestors}" if DEBUG
+				#puts "ancestors is #{puta ancestors}" 
 				axe_helper( path, ancestors)
 
 			when "child"
 				results = []
-				#puts "PATH IS '#{path}'" if DEBUG
+				#puts "PATH IS '#{path}'" 
 				p = path
 				nodeset.each do |element|
 					if element.kind_of? Element
-						#puts "YIELDING CHILDREN OF #{element.name if element.kind_of? Element}" if DEBUG
+						#puts "YIELDING CHILDREN OF #{element.name if element.kind_of? Element}" 
 						p,r = axe_helper( path, element.to_a )
-						#puts "P IS NOW '#{p}'" if DEBUG
+						#puts "P IS NOW '#{p}'" 
 						results += r
 					end
 				end
-				#puts "CHILD: P IS NOW '#{p}'" if DEBUG
+				#puts "CHILD: P IS NOW '#{p}'" 
 				[p, results]
 
 			when "attribute"
 				results = []
-				#puts "ATTRIBUTE: NODESET SIZE IS #{nodeset.size}" if DEBUG
+				#puts "ATTRIBUTE: NODESET SIZE IS #{nodeset.size}" 
 				p = ''
 				nodeset.each do |element|
 					p,r=axe_helper(path,element.attributes.values.flatten) if element.kind_of? Element
-					#puts "#{path} => #{p}" if DEBUG
+					#puts "#{path} => #{p}" 
+					next unless r
 					results += r
 					results << nil if r.size < 1
 				end
-				#puts "ATTRIBUTE: P IS NOW '#{p}'" if DEBUG
-				#puts "results is now #{puta results}" if DEBUG
-				results.collect!{ |a| a.value if a }
+				#puts "ATTRIBUTE: P IS NOW '#{p}'" 
+				#puts "results is now #{puta results}" 
+				#results.collect!{ |a| a.value if a }
 				[p,results]
 
 			when "parent"
@@ -421,22 +453,22 @@ module REXML
 
 		def axe_helper path, nodeset
 			# NodeTest returns a true/false 1-1 map of nodeset
-			#puts "axe_helper with nodeset #{puta nodeset}" if DEBUG
+			#puts "axe_helper with nodeset #{puta nodeset}" 
 			path, node_test = NodeTest( path, nodeset )
-			#puts "NodeTest returned '#{path}' and #{puta node_test}" if DEBUG
-			#puts "Nodeset is now #{puta nodeset}" if DEBUG
+			#puts "NodeTest returned '#{path}' and #{puta node_test}" 
+			#puts "Nodeset is now #{puta nodeset}" 
 			join( nodeset, node_test ) {|a,o,b| a if b}
-			#puts "After join, nodeset is #{puta nodeset}" if DEBUG
+			#puts "After join, nodeset is #{puta nodeset}" 
 			nodeset.compact!
 			# Predicate returns a true/false 1-1 map  of nodeset
 			if path[0] == ?[
 				path, predicate = Predicate( path, nodeset )
-				#puts "PREDICATE RETURNED '#{path}' and #{puta predicate}" if DEBUG
-				#puts "Joining with #{puta nodeset}" if DEBUG
+				#puts "PREDICATE RETURNED '#{path}' and #{puta predicate}" 
+				#puts "Joining with #{puta nodeset}" 
 				join( nodeset, predicate ) {|a,o,b| a if b}
 				nodeset.compact!
 			end
-			#puts "AXE HELPER RETURNING '#{path}'" if DEBUG
+			#puts "AXE HELPER RETURNING '#{path}'" 
 			[path, nodeset]
 		end
 
@@ -460,18 +492,18 @@ module REXML
 		#	| NODE_TYPE '(' ')'															NodeType
 		#	| PI '(' LITERAL ')'														PI
 		#		| '[' expr ']'																Predicate
-		NCNAMETEST= /^(#{Namespace::NCNAME_STR}):\*/u
+		NCNAMETEST= /^(#{NCNAME_STR}):\*/u
 		QNAME 		= Namespace::NAMESPLIT
 		NT				= /^comment|text|processing-instruction|node$/
 		NODE_TYPE	= /^(comment|text|processing-instruction|node)\(\)/
 		PI				= /^processing-instruction\(/
 		def NodeTest path, nodeset
 			res = nil
-			#puts "NodeTest with '#{path}' and #{puta nodeset}" if DEBUG
+			#puts "NodeTest with '#{path}' and #{puta nodeset}" 
 			case path
 			when /^\*/
 				path = $'
-				res = nodeset.collect {|n| n.kind_of? Element}
+				res = nodeset.collect {|n| Element===n || Attribute===n}
 			when NODE_TYPE
 				type = $1
 				path = $'
@@ -493,8 +525,10 @@ module REXML
 				p =~ LITERAL
 				literal = $1
 				p = p[$&.length..-1]
-				raise ParseException.new("Missing ')' after processing instruction") if path[0]!=?)
-				res = processing_instruction(literal)
+				raise ParseException.new("Missing ')' after processing instruction") if p[0]!=?)
+				res = nodeset.collect { |n|
+					true if n.kind_of? Instruction and n.target == literal
+				}
 			when NCNAMETEST
 				#puts "NCNAMETEST #{path}"
 				#puts "PREFIX = #$1"
@@ -511,7 +545,7 @@ module REXML
 				name = $2
 				path = $'
 				prefix = "" unless prefix
-				#puts "QNAME #{prefix}:#{name}, '#{path}'" if DEBUG
+				#puts "QNAME #{prefix}:#{name}, '#{path}'" 
 				res = nodeset.collect { |n|
 					n.kind_of? Namespace and name==n.name and matching_namespace?(n, prefix)
 				}
@@ -522,13 +556,13 @@ module REXML
 		def matching_namespace?( node, prefix )
 			prefix = node.prefix if node.kind_of? Attribute and prefix == ""
 			nscontext = (@namespaces.nil? ? node.namespace(prefix) : @namespaces[prefix] )
-			#print " -- matching_namespace? #{node.namespace} == #{nscontext}" if DEBUG
+			#print " -- matching_namespace? #{node.namespace} == #{nscontext}" 
 			node.namespace == nscontext
 		end
 
 		# Filters the supplied nodeset on the predicate(s)
 		def Predicate path, nodeset
-			#puts "Predicate '#{path}'" if DEBUG
+			#puts "Predicate '#{path}'" 
 			return nil unless path[0] == ?[
 			predicates = []
 			while path[0] == ?[
@@ -541,22 +575,21 @@ module REXML
 			predicates.each do |expr|
 				subresults = []
 				nodes.each do |node|
-					#puts "PREDICATE FILTERING #{node.name if node.kind_of? Element}" if DEBUG
+					#puts "PREDICATE FILTERING #{node.name if node.kind_of? Element}" 
 					ind = nodes.index(node)+1
 					Functions.node = node
 					Functions.pair = [ ind, nodes.size ]
 					Functions.namespace_context = node
 					p, r = OrExpr(expr, [node])
 					r = r[0] if r.kind_of? Array
-					if r.to_s =~ /^\d+$/
-						r = r.to_i
-						#puts "COMPARING #{r} to #{ind}"
+					if r.to_s =~ /^\d+(\.\d+)?$/
+						r = r.to_s.to_f.round
 						r = (r == ind)
 					end
-					#puts "PREDICATE ON NODE #{node.name if node.kind_of? Element}" if DEBUG
-					#puts "PREDICATE RESULTS ARE #{r}" if DEBUG
+					#puts "PREDICATE ON NODE #{node.name if node.kind_of? Element}" 
+					#puts "PREDICATE RESULTS ARE #{r}" 
 					subresults << r
-					#puts "RESULT SET IS #{puta subresults}" if DEBUG
+					#puts "RESULT SET IS #{puta subresults}" 
 				end
 				join( nodes, subresults ) {|a,o,b| a if b}
 				join( mapping, subresults ) {|a,o,b| a if b}
@@ -567,7 +600,7 @@ module REXML
 			results = []
 			mapping.each{|ind| results[ind] = true}
 			results << false while results.size < nodeset.size
-			#puts "PREDICATE RETURNING #{puta results}" if DEBUG
+			#puts "PREDICATE RETURNING #{puta results}" 
 			[path, results]
 		end
 
@@ -643,19 +676,20 @@ module REXML
 			rv << arry.name if arry.kind_of? Element
 			arry.each { |n|
 				rv << "\n\t"
+				rv << n.to_s if n.kind_of? Numeric
+				rv << ' '
 				rv << n.type.to_s
-				rv << " "
+				rv << ' '
 				rv << n.name if n.kind_of? Element
 				rv << "'#{n}'" if n.kind_of? String or n.kind_of? Fixnum
 				rv << " "
 				rv << n.attributes.values.collect{|v|v.to_s}.to_s if n.kind_of? Element
-				rv << "\t#{arry.id}"
 			}
 			rv
 		end
 
 		def parse_args( string )
-			#puts "parse_args '#{string}'" if DEBUG
+			#puts "parse_args '#{string}'" 
 			arguments = []
 			ind = 0
 			depth = 1
@@ -689,7 +723,7 @@ module REXML
 		end
 
 		def recurse( nodeset, &block )
-			#puts "RECURSE" if DEBUG
+			#puts "RECURSE" 
 			nodeset.each do |node|
 				yield node
 				recurse( node, &block ) if node.kind_of? Element
@@ -697,23 +731,77 @@ module REXML
 		end
 
 		def equality_relational_compare( set1, op, set2 )
-			#puts "COMPARING #{set1} #{op} #{set2}" if DEBUG
+			#puts "COMPARING #{set1} (#{set1.type}) #{op} #{set2} (#{set2.type})" 
 			if set1.kind_of? Array and set2.kind_of? Array
-				# Compare two sets
-				set1.each{|i1| set2.each{|i2| return true if i1.to_s.send(op, i2.to_s) } }
-				# Return
-			else
-				if set1.kind_of? Array
-					set2 = set2.to_s
-					set1.each{|v| return true if v.to_s.send(op,set2)}
-				elsif set2.kind_of? Array
-					set1 = set1.to_s
-					set2.each{|v| return true if set1.send(op,v.to_s)}
+				if set1.size == 1 and set2.size == 1
+					set1 = set1[0]
+					set2 = set2[0]
 				else
-					# Compare two vals
-					return [set1.to_s.send(op, set2.to_s)]
+					# Compare two sets
+					set1.each{|i1| 
+						set2.each{|i2| 
+							return true if i1.to_s.send(op, i2.to_s) 
+						} 
+					}
+					return false
 				end
 			end
+			#puts "COMPARING VALUES"
+			# If one is nodeset and other is number, compare number to each item
+			# in nodeset s.t. number op number(string(item))
+			# If one is nodeset and other is string, compare string to each item
+			# in nodeset s.t. string op string(item)
+			# If one is nodeset and other is boolean, compare boolean to each item
+			# in nodeset s.t. boolean op boolean(item)
+			if set1.kind_of? Array or set2.kind_of? Array
+				if set1.kind_of? Array
+					a = set1
+					b = set2.to_s
+				else
+					a = set2
+					b = set1.to_s
+				end
+
+				block = nil
+				case b
+				when 'true', 'false'
+					b = Functions::boolean( b )
+					block = proc{|v| return true if Functions::boolean(v).send(op,b)}
+				when /^\d+(\.\d+)?$/
+					b = Functions::number( b )
+					block = proc{|v| return true if Functions::number(v).send(op,b)}
+				else
+					b = Functions::string( b )
+					block = proc{|v| return true if Functions::string(v).send(op,b)}
+				end
+				a.each( &block )
+			else
+				# If neither is nodeset,
+				#   If op is = or !=
+				#     If either boolean, convert to boolean
+				#     If either number, convert to number
+				#     Else, convert to string
+				#   Else
+				#     Convert both to numbers and compare
+				if op == '==' or op == '!='
+					set1 = set1.to_s
+					set2 = set2.to_s
+					if set1 == 'true' or set1 == 'false' or set2 == 'true' or set2 == 'false'
+						set1 = Functions::boolean( set1 )
+						set2 = Functions::boolean( set2 )
+					elsif set1 =~ /^\d+(\.\d+)?$/ or set2 =~ /^\d+(\.\d+)?$/
+						set1 = Functions::number( set1 )
+						set2 = Functions::number( set2 )
+					else
+						set1 = Functions::string( set1 )
+						set2 = Functions::string( set2 )
+					end
+				else
+					set1 = Functions::number( set1 )
+					set2 = Functions::number( set2 )
+				end
+				return [set1.send(op, set2)]
+			end
 			return false
 		end
 
@@ -747,9 +835,3 @@ module REXML
 		end
 	end
 end
-
-class String
-	def ne( other )
-		self != other
-	end
-end
